package bootcampdia8.FilmCrud.controller;

import bootcampdia8.FilmCrud.FilmHelper;
import bootcampdia8.FilmCrud.model.Film;
import bootcampdia8.FilmCrud.model.Language;
import bootcampdia8.FilmCrud.repository.ActorRepository;
import bootcampdia8.FilmCrud.repository.FilmRepository;
import bootcampdia8.FilmCrud.repository.LanguageRepository;
import org.apache.jasper.compiler.JspUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.persistence.metamodel.Bindable;
import javax.servlet.http.HttpServletRequest;


@Controller
public class FilmController {

    private FilmRepository filmRepository;
    private LanguageRepository languageRepository;
    private ActorRepository actorRepository;

    @Autowired
    public FilmController(FilmRepository filmRepository,
                          LanguageRepository languageRepository,
                          ActorRepository actorRepository){
        this.filmRepository = filmRepository;
        this.languageRepository = languageRepository;
        this.actorRepository = actorRepository;
    }

    @GetMapping("/films")
    public ModelAndView serveMainPage(HttpServletRequest request){
        String id = request.getParameter("id");
        long idl;
        FilmHelper filmHelper = new FilmHelper();
        ModelAndView mv = new ModelAndView("main", "film", filmHelper);
        if(id!=null){
            Film film;
            idl = Long.parseLong(id);
            film = filmRepository.getById(idl);
            filmHelper.setId(idl);
            filmHelper.setTitle(film.getTitle());
            filmHelper.setDescription(film.getDescription());
            filmHelper.setReleaseYear(String.valueOf(film.getReleaseYear()));
            filmHelper.setDuration(String.valueOf(film.getDuration()));
            filmHelper.setLanguage(String.valueOf(film.getLanguage()));
            mv.addObject("lenguaje", film.getLanguage().getName());
            mv.addObject("actors", film.getActors());
        }
        mv.addObject("films", filmRepository.findAll(Sort.by(Sort.Direction.ASC, "id")));
        return mv;
    }

    @PostMapping("/addFilm")
    public String addFilm(HttpServletRequest request, @ModelAttribute("film") FilmHelper filmHelper, BindingResult bindingResult, ModelMap modelMap) {
        Film film = new Film();
        if (bindingResult.hasErrors()) {
            return "ERRO";
        }
        if (request.getParameter("action").equals("update")) {
            System.out.println(request.getParameter("id"));
            film = filmRepository.getById(Long.parseLong(request.getParameter("id")));
        }
        film.setTitle(filmHelper.getTitle());
        film.setDescription(filmHelper.getDescription());
        film.setReleaseYear(Long.parseLong(filmHelper.getReleaseYear()));
        film.setDuration(Long.parseLong(filmHelper.getDuration()));
        film.setLanguage(languageRepository.getById(Long.parseLong(filmHelper.getLanguage())));
        filmRepository.save(film);
        return "redirect:/films";
    }

    @GetMapping("/eliminar/{filmId}/")
    public String eliminarFilm(@PathVariable("filmId") String id){
        filmRepository.deleteById(Long.parseLong(id));
        return "redirect:/films";
    }

    @GetMapping("/films/{langId}")
    public ModelAndView listFilmsByLanguages(@PathVariable("langId") long langId){
        ModelAndView mv = new ModelAndView("peliculasConLenguajeX");
//        mv.addObject("lenguajes", filmRepository.findByLanguage(languageRepository.getById(langId)));
        mv.addObject("peliculas", filmRepository.obtenerPorLenguaje(langId));
        return mv;
    }

    @GetMapping("/films/getByTitle/{title}")
    public ModelAndView getDetailFilmByTitle(@PathVariable("title") String title){
        ModelAndView mv = new ModelAndView("filmDetail");
        mv.addObject("film", filmRepository.findByTitle(title));
        return mv;
    }

    @GetMapping("/filmsAndActors")
    public String getFilmsAndActors(HttpServletRequest request, Model model){
        ModelAndView mv = new ModelAndView();
        model.addAttribute("list", filmRepository.joinFilmsAndActors());
        return "filmsAndActors";
    }

    @GetMapping("/filmsAndLanguages")
    public String getFilmsAndLanguages(HttpServletRequest request, Model model){
        ModelAndView mv = new ModelAndView();
        model.addAttribute("list", filmRepository.joinFilmsAndLanguages());
        return "filmAndLanguages";
    }

}
